#include <ros/ros.h>
#include <image_transport/image_transport.h>

#include <cv_bridge/cv_bridge.h>
#include <opencv2/core/core.hpp>

#include <std_msgs/Float32.h>

#include <sensor_msgs/image_encodings.h>



class TestNode
{
  public:
    TestNode() :
      nh_(),
      it_ (nh_),
      example_pub_ (nh_.advertise<std_msgs::Float32> ("number", 1, true)),
      example_sub_ (nh_.subscribe ("number", 1, &TestNode::example_callback, this)),
      img_pub_ (it_.advertise ("image", 1)) {
      ROS_INFO_STREAM ("Ctor");
      
      std_msgs::Float32 msg;
      msg.data = 3.1415;
      example_pub_.publish (msg);
    }
    
    void example_callback (std_msgs::Float32 msg) {
      if (! ros::param::getCached ("parameter", param_)) {
        ros::param::set ("parameter", 5);
      }
      
      ROS_INFO_STREAM ("Received message: " << msg.data << " Current param: " << param_);
      
      cv::Mat image (480, 640, CV_8UC3, cv::Scalar (255, 255, 255));
      
      cv::circle (image,
                  cv::Point (240, 240),
                  220,
                  cv::Scalar (0, 0, 0),
                  5);
                  
      cv::line (image,
                cv::Point (240, 240),
                cv::Point (240 + 220 * cos (-param_), 240 + 220 * sin (-param_)),
                cv::Scalar (0, 0, 0),
                3);
                
      cv_bridge::CvImage out_msg;
      out_msg.header.stamp = ros::Time::now();
      out_msg.encoding = sensor_msgs::image_encodings::BGR8;
      out_msg.image = image;
      img_pub_.publish (out_msg.toImageMsg());
    }
    
    ~TestNode() {
    }
    
  private:
    ros::NodeHandle nh_;
    image_transport::ImageTransport it_;
    ros::Publisher example_pub_;
    ros::Subscriber example_sub_;
    image_transport::Publisher img_pub_;
    
    int param_;
};



int
main (int argc, char** argv)
{
  ros::init (argc, argv, "test_node");
  
  TestNode test_node;
  
  ros::spin();
  
  return 0;
}
